//
//  XYMarkerView.swift
//  ChartsDemo
//  Copyright © 2016 dcg. All rights reserved.
//

import Foundation
import Charts

open class XYMarkerView: BalloonMarker
{
    @objc open var xAxisValueFormatter: IAxisValueFormatter?
    fileprivate var yFormatter = NumberFormatter()
    
    @objc public init(color: UIColor, font: UIFont, textColor: UIColor, insets: UIEdgeInsets,
                      xAxisValueFormatter: IAxisValueFormatter)
    {
        super.init(color: color, font: font, textColor: textColor, insets: insets)
        self.xAxisValueFormatter = xAxisValueFormatter
    }
    
    open override func refreshContent(entry: ChartDataEntry, highlight: Highlight)
    {
        var dataType : String
        var dataUnit : String
        switch self.chartType {
        case 0:
            dataType = "Clients: "
            dataUnit = ""
            break
        case 1:
            dataType = "Traffic: "
            dataUnit = " KBPS"
            enableFractionalValue()
            break
        case 2:
            dataType = "Traffic: "
            dataUnit = " MBPS"
            enableFractionalValue()
            break
        case 3:
            dataType = "Traffic: "
            dataUnit = " GBPS"
            enableFractionalValue()
            break
        default:
            dataType = "Value: "
            dataUnit = ""
        }
        setLabel("  " + xAxisValueFormatter!.stringForValue(entry.x, axis: nil) + "\n  " + dataType + yFormatter.string(from: NSNumber(floatLiteral: entry.y))! + dataUnit)
    }
    
    /// Set yAxis values to be formatted with fractional part
    open func enableFractionalValue() {
        yFormatter.maximumFractionDigits = 1;
        yFormatter.minimumFractionDigits = 1;
        yFormatter.minimumIntegerDigits = 1;
    }
}

